#include "para.h"
#include "list.h"

static struct list_head close_on_fork_list;
static int initialized;

struct close_on_fork {
	int fd;
	struct list_head node;
};

void add_close_on_fork_list(int fd)
{
	struct close_on_fork *cof = para_malloc(sizeof(struct close_on_fork));

	if (!initialized) {
		INIT_LIST_HEAD(&close_on_fork_list);
		initialized = 1;
	}
	cof->fd = fd;
	list_add(&cof->node, &close_on_fork_list);
}

void del_close_on_fork_list(int fd)
{
	struct close_on_fork *cof, *tmp;

	if (!initialized)
		return;
	list_for_each_entry_safe(cof, tmp, &close_on_fork_list, node) {
		if (fd != cof->fd)
			continue;
		list_del(&cof->node);
		free(cof);
	}
}

void close_listed_fds(void)
{
	struct close_on_fork *cof;

	if (!initialized)
		return;
	list_for_each_entry(cof, &close_on_fork_list, node) {
		PARA_DEBUG_LOG("closing fd %d\n", cof->fd);
		close(cof->fd);
	}
}
