/*
 * Copyright (C) 2006 Andre Noll <maan@systemlinux.org>
 *
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 *
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */
#include "gcc-compat.h"
#include "para.h"

#include "recv.h"

extern int http_recv_init(struct receiver *r);
extern int ortp_recv_init(struct receiver *r);
void (*crypt_function_recv)(unsigned long len, const unsigned char *indata, unsigned char *outdata) = NULL;
void (*crypt_function_send)(unsigned long len, const unsigned char *indata, unsigned char *outdata) = NULL;

DEFINE_RECEIVER_ARRAY(receivers);
static void *parse_receiver_args(int receiver_num, char *options)
{
	struct receiver *r = &receivers[receiver_num];
	char **argv;
	int argc, i;
	void *conf;


//	PARA_DEBUG_LOG("%s, options: %s\n", r->name,
//		options? options : "(none)");
	if (options) {
//		PARA_DEBUG_LOG("%s options: %s\n", name, options);
		argc = split_args(options, &argv, ':');
//		PARA_DEBUG_LOG("argc = %d, argv[0]: %s\n", fn->argc, fn->argv[0]);
		for (i = argc; i >= 0; i--)
			argv[i + 1] = argv[i];
		argc += 2;
	} else {
		argc = 1;
		argv = para_malloc(2 * sizeof(char*));
		argv[0] = NULL;
		argv[1] = NULL;
	}
	conf = r->parse_config(argc, argv);
	if (!conf) {
		for (i = 0; i < argc; i++)
			free(argv[i]);
		free(argv);
		return NULL;
	}
	return conf;
}

void *check_receiver_arg(char *ra, int *receiver_num)
{
	int j;

//	PARA_DEBUG_LOG("checking %s\n", ra);
	for (j = 0; receivers[j].name; j++) {
		const char *name = receivers[j].name;
		size_t len = strlen(name);
		char c;
		if (strlen(ra) < len)
			continue;
		if (strncmp(name, ra, len))
			continue;
		c = ra[len];
		if (c && c != ':')
			continue;
		if (c && !receivers[j].parse_config)
			return NULL;
		*receiver_num = j;
		return parse_receiver_args(j, c? ra + len + 1: NULL);
	}
	PARA_ERROR_LOG("%s", "receiver not found:");
	return NULL;
}
